import htmls
from django import test
from django_cradmin import uicontainer


class TestMessagesContainer(test.TestCase):
    def test_do_not_render_if_empty(self):
        container = uicontainer.messagescontainer.MessagesContainer(
            test_css_class_suffixes_list=['messages']
        )\
            .bootstrap()
        selector = htmls.S(container.render())
        self.assertFalse(selector.exists('.test-messages'))

    def test_render_if_not_empty(self):
        container = uicontainer.messagescontainer.MessagesContainer(
            test_css_class_suffixes_list=['messages']
        )\
            .add_info(text='test')\
            .bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('.test-messages'))

    def test_css_classes_default(self):
        container = uicontainer.messagescontainer.MessagesContainer(
            test_css_class_suffixes_list=['messages']
        )\
            .add_info(text='test')\
            .bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('.test-messages').cssclasses_set,
                         {'blocklist', 'blocklist--tight', 'test-messages'})

    def test_add_message(self):
        container = uicontainer.messagescontainer.MessagesContainer(
            test_css_class_suffixes_list=['messages']
        )\
            .add_message(level='info', text='test')\
            .bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('.blocklist__item').alltext_normalized,
                         'test')

    def test_add_warning(self):
        container = uicontainer.messagescontainer.MessagesContainer(
            test_css_class_suffixes_list=['messages']
        )\
            .add_warning(text='test')\
            .bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('.blocklist__item.blocklist__item--warning').alltext_normalized,
                         'test')

    def test_add_success(self):
        container = uicontainer.messagescontainer.MessagesContainer(
            test_css_class_suffixes_list=['messages']
        )\
            .add_success(text='test')\
            .bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('.blocklist__item.blocklist__item--success').alltext_normalized,
                         'test')

    def test_add_info(self):
        container = uicontainer.messagescontainer.MessagesContainer(
            test_css_class_suffixes_list=['messages']
        )\
            .add_info(text='test')\
            .bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('.blocklist__item.blocklist__item--info').alltext_normalized,
                         'test')



from django import test

from django_cradmin.templatetags import cradmin_tags


class TestCradminTestCssClass(test.TestCase):
    def test_test_css_classes_disabled(self):
        with self.settings(DJANGO_CRADMIN_INCLUDE_TEST_CSS_CLASSES=False):
            self.assertEqual('', cradmin_tags.cradmin_test_css_class('my-css-class'))

    def test_test_css_classes_enabled(self):
        with self.settings(DJANGO_CRADMIN_INCLUDE_TEST_CSS_CLASSES=True):
            self.assertEqual('  test-my-css-class  ',
                             cradmin_tags.cradmin_test_css_class('my-css-class'))



from __future__ import unicode_literals
from builtins import range
from builtins import object
from django.core.exceptions import ValidationError
from django.db import models

from django_cradmin.utils.nulls_last_queryset import NullsLastManager, NullsLastQuerySet


class SortableQuerySetBase(NullsLastQuerySet):
    """
    QuerySet for :class:`.SortableManagerBase`.

    You must use this as a base class if you want to create
    a custom queryset class for models extending :class:`.SortableBase`.
    """


class SortableManagerBase(NullsLastManager):
    """
    Manager for :class:`.SortableBase`.
    """
    parent_attribute = None

    def _get_filtered_by_parentobject_queryset(self, parentobject):
        filter_kwargs = {self.parent_attribute: parentobject}
        return self.get_queryset().filter(**filter_kwargs).order_by('sort_index')

    def _get_siblings_queryset(self, item, ignore_none=True):
        """
        Get the queryset for all the items with the same parent as the given
        item.
        """
        parentobject = getattr(item, self.parent_attribute)
        queryset = self._get_filtered_by_parentobject_queryset(parentobject)
        if ignore_none:
            queryset = queryset.exclude(sort_index=None)
        return queryset

    def _get_last_sortindex_within_parentobject(self, item):
        lastitem = self._get_siblings_queryset(item).last()
        if lastitem:
            return lastitem.sort_index + 1
        else:
            return 0

    def set_newitem_sort_index_to_last(self, item):
        """
        Sets ``item.sort_index`` to the sort_index of the last item
        in the parent + 1. Does not save.

        ONLY USE THIS FOR NEWLY CREATED ITEMS.
        """
        if item.sort_index is not None or item.pk is not None:
            raise ValueError('set_newitem_sort_index_to_last should only be used '
                             'when creating new items. sort_index or pk is something '
                             'other than None, which indicates that this is not a new '
                             'item - thus the ValueError.')
        item.sort_index = self._get_last_sortindex_within_parentobject(item)

    def __increase_sort_index_in_range(self, queryset, items, from_index, to_index, distance=1):
        """
        Used by sort_before to increase the sort_index of all items in the given range
        """
        increase_index_for_items = [itm.id for itm in items[from_index:to_index]]
        queryset.filter(id__in=increase_index_for_items).update(sort_index=models.F('sort_index') + distance)

    def __decrease_sort_index_in_range(self, queryset, items, from_index, to_index, distance=1):
        """
        Used by sort_before to decrease the sort_index of all items in the given range
        """
        decrease_index_for_items = [itm.id for itm in items[from_index:to_index]]
        queryset.filter(id__in=decrease_index_for_items).update(sort_index=models.F('sort_index') - distance)

    def __fix_sort_order(self, item, sort_before_id):
        """
        Iterate over all the items and remove any gaps, fix any duplicate
        sort indexes and find the correct sort index for the item.

        Parameters:
            item: The item to insert in the list.
            sort_before_id: ID of the item that you want to insert ``item`` before.

        Returns:
            A tuple (itemsqueryset, sort_before_index, original_item_index):

            - ``itemsqueryset`` is a queryset with all the items within the
              :obj:`.parentobject`, sorted by ``sort_index``, without any gaps
              or duplicate sort indexes.
            - ``detected_item_sort_index``: The detected new sort index for the item.
              If this is ``None``, the item should be sorted last.
            - ``original_item_index``: The sort index of the ``item``. This may not
              be the same as ``item.sort_index`` if the item has been moved up or
              down because of gaps or duplicate sort indexes.
        """
        itemsqueryset = self._get_siblings_queryset(item)
        detected_item_sort_index = None
        original_item_index = None
        for index in range(0, len(itemsqueryset)):
            current_item = itemsqueryset[index]
            if index < current_item.sort_index:
                # Found gap, need to move rest of list <size-of-gap> step(s) down
                self.__decrease_sort_index_in_range(itemsqueryset, itemsqueryset, index,
                                                    len(itemsqueryset),
                                                    current_item.sort_index - index)
                itemsqueryset = self._get_siblings_queryset(item).all()
            if index > current_item.sort_index:
                # Found duplicate sort_index, need to move rest of list one step up
                self.__increase_sort_index_in_range(itemsqueryset, itemsqueryset, index,
                                                    len(itemsqueryset))
                itemsqueryset = self._get_siblings_queryset(item).all()

            if item.id == current_item.id:
                original_item_index = current_item.sort_index
            elif sort_before_id is not None and current_item.id == sort_before_id:
                detected_item_sort_index = current_item.sort_index
        return itemsqueryset, detected_item_sort_index, original_item_index

    def __sort_last(self, itemsqueryset, item, original_item_index):
        if original_item_index is not None:
            # fill gap left when moving item to end
            self.__decrease_sort_index_in_range(itemsqueryset, itemsqueryset,
                                                original_item_index + 1, len(itemsqueryset))
            item.sort_index = len(itemsqueryset) - 1
        else:
            item.sort_index = len(itemsqueryset)
        item.save()

    def __sort_not_last(self, itemsqueryset, item, detected_item_sort_index, original_item_index):
        if original_item_index is None:
            # new item, move rest of list one step up, and place the new item
            self.__increase_sort_index_in_range(itemsqueryset,
                                                itemsqueryset,
                                                detected_item_sort_index,
                                                len(itemsqueryset))
            item.sort_index = detected_item_sort_index
        elif original_item_index < detected_item_sort_index:
            # Move up, and fill gap left behind by moving items in the gap down
            self.__decrease_sort_index_in_range(itemsqueryset,
                                                itemsqueryset,
                                                original_item_index + 1,
                                                detected_item_sort_index)
            item.sort_index = detected_item_sort_index - 1
        elif original_item_index > detected_item_sort_index:
            # Move down, and fill/create gap by moving other objects up
            self.__increase_sort_index_in_range(itemsqueryset,
                                                itemsqueryset,
                                                detected_item_sort_index,
                                                original_item_index)
            item.sort_index = detected_item_sort_index
        else:
            # Item is already in the correct place
            # - Return without saving
            return
        item.save()

    def sort_before(self, item, sort_before_id):
        """
        Sort a given item before the item with id `sort_before_id`,
        or last if `sort_before_id` is ``None``.

        Fetches all items in the same container, and makes changes in the ordering.
        Only the required updates are made.
        """
        itemsqueryset, detected_item_sort_index, original_item_index = self.__fix_sort_order(item, sort_before_id)

        if detected_item_sort_index is None:
            self.__sort_last(itemsqueryset=itemsqueryset, item=item,
                             original_item_index=original_item_index)
        else:
            self.__sort_not_last(itemsqueryset=itemsqueryset,
                                 item=item,
                                 detected_item_sort_index=detected_item_sort_index,
                                 original_item_index=original_item_index)

    def sort_last(self, item):
        """
        Just a shortcut for::

            self.sort_before(item, sort_before_id=None)
        """
        self.sort_before(item, sort_before_id=None)

    def get_query_set(self):
        return SortableQuerySetBase(self.model, using=self._db)


def validate_sort_index(value):
    if value < 0:
        raise ValidationError(u'Sort index must be 0 or higher.')


class SortableBase(models.Model):
    """
    Used with :class:`.SortableManagerBase` to make models sortable.
    """

    #: Sort index - ``0`` or higher.
    sort_index = models.PositiveIntegerField(
        blank=True, null=True,
        default=None,
        verbose_name='index',
        validators=[validate_sort_index]
    )

    class Meta(object):
        abstract = True



from django.views.generic import TemplateView

from django_cradmin import javascriptregistry


class StandaloneBaseTemplateView(TemplateView, javascriptregistry.viewmixin.StandaloneBaseViewMixin):
    """
    Base template view that you should use instead of :class:`django.views.generic.TemplateView`
    if you extend the ``django_cradmin/standalone-base.django.html`` template.
    """
    template_name = 'django_cradmin/standalone-base.django.html'

    def get_context_data(self, **kwargs):
        context = super(StandaloneBaseTemplateView, self).get_context_data(**kwargs)
        self.add_javascriptregistry_component_ids_to_context(context=context)
        return context


class WithinRoleTemplateView(TemplateView, javascriptregistry.viewmixin.WithinRoleViewMixin):
    """
    Base template view that you should use instead of :class:`django.views.generic.TemplateView`
    if you extend the ``django_cradmin/base.django.html`` template.
    """
    template_name = 'django_cradmin/base.django.html'

    def get_context_data(self, **kwargs):
        context = super(WithinRoleTemplateView, self).get_context_data(**kwargs)
        self.add_javascriptregistry_component_ids_to_context(context=context)
        return context



from . import container


class Nothing(container.AbstractContainerRenderable):
    """
    Does not render anything.

    Useful in cases where you do not want to render anything.
    We use this instead of ``None`` to avoid having to handle
    ``None`` values everywhere.

    Typical use case is the help_text_renderable kwarg for
    :class:`django_cradmin.uicontainer.field.FieldWrapper`.
    If you want to disable the help text, you can not set the kwarg
    to ``None``, because that would just fall back to using the
    default help text renderable, so you use class instead.
    """

    @property
    def should_render(self):
        return False



from . import container
from . import convenience


class BlocklistItemTitle(convenience.AbstractWithOptionalEscapedText):
    """
    Renders :class:`.BlocklistItem` title.

    The HTML element has the ``blocklist__itemtitle`` css class.
    """
    def get_default_bem_block_or_element(self):
        return 'blocklist__itemtitle'


class BlocklistItem(convenience.AbstractWithOptionalParagraphWithEscapedText):
    """
    Renders an item suitable to be a direct child of :class:`.Blocklist`.

    The HTML element has the ``blocklist__item`` css class.
    """
    def get_default_bem_block_or_element(self):
        return 'blocklist__item'


class Blocklist(container.AbstractContainerRenderable):
    """
    Renders a list of block elements styled with the ``blocklist`` css class.

    Examples:

        Simple example::

            from django_cradmin import uicontainer

            uicontainer.blocklist.Blocklist(
                children=[
                    uicontainer.blocklist.BlocklistItem(text='Item 1'),
                    uicontainer.blocklist.BlocklistItem(text='Item 2')
                ]
            )

        Using variants of both the list and the items::

            from django_cradmin import uicontainer

            uicontainer.blocklist.Blocklist(
                bem_variant_list=['tight']
                children=[
                    uicontainer.blocklist.BlocklistItem(text='Success !', bem_variant_list=['success']),
                    uicontainer.blocklist.BlocklistItem(text='Warning :(', bem_variant_list=['warning'])
                ]
            )

        More complex blocklist items::

            from django_cradmin import uicontainer

            uicontainer.blocklist.Blocklist(
                children=[
                    uicontainer.blocklist.BlocklistItem(
                        children=[
                            uicontainer.blocklist.BlocklistItemTitle(html_tag='h3')
                            uicontainer.semantic.Paragraph(text='Hello world')
                        ]
                    ),
                    uicontainer.blocklist.BlocklistItem(
                        children=[
                            uicontainer.blocklist.BlocklistItemTitle(html_tag='h3',
                                bem_variant_list=['xlarge'])
                            uicontainer.semantic.Paragraph(text='Hello world 2')
                        ]
                    ),
                ]
            )
    """
    def get_default_bem_block_or_element(self):
        return 'blocklist'



from __future__ import unicode_literals
from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.module_loading import import_string

from django.views.generic import FormView

from django_cradmin import javascriptregistry
from django_cradmin.apps.cradmin_activate_account.utils import ActivationEmail


class BeginRegisterAccountView(FormView, javascriptregistry.viewmixin.StandaloneBaseViewMixin):
    template_name = 'cradmin_register_account/begin.django.html'

    def get_form_class(self):
        if self.form_class:
            return self.form_class
        else:
            return import_string(settings.DJANGO_CRADMIN_REGISTER_ACCOUNT_FORM_CLASS)

    def get_context_data(self, **kwargs):
        context = super(BeginRegisterAccountView, self).get_context_data(**kwargs)
        self.add_javascriptregistry_component_ids_to_context(context=context)
        context['DJANGO_CRADMIN_SITENAME'] = settings.DJANGO_CRADMIN_SITENAME
        return context

    def get_success_url(self):
        return reverse('cradmin-register-account-email-sent')

    def get_next_url(self):
        """
        Get the next url to go to after the account has been activated.

        Defaults to the ``DJANGO_CRADMIN_REGISTER_ACCOUNT_REDIRECT_URL``, falling back to
        the ``LOGIN_URL`` setting.
        """
        if 'next' in self.request.GET:
            return self.request.GET['next']
        else:
            return getattr(settings, 'DJANGO_CRADMIN_REGISTER_ACCOUNT_REDIRECT_URL', settings.LOGIN_URL)

    def send_activation_email(self, user):
        activation_email = ActivationEmail(
            request=self.request,
            user=user,
            next_url=self.get_next_url())
        activation_email.send()

    def form_valid(self, form):
        user = form.save()
        self.send_activation_email(user)
        return super(BeginRegisterAccountView, self).form_valid(form)



from . import container
from . import messagescontainer


class Form(container.AbstractContainerRenderable):
    """
    Renderable for a ``<form>``.
    """
    template_name = 'django_cradmin/uicontainer/form.django.html'

    def __init__(self, form, action=None, method=None, messages_container=None, **kwargs):
        """

        Args:
            form: The Django form to render.
            action: The action attribute of the HTML form.
                If this is ``None`` (the default), we will
                use :meth:`django.http.HttpRequest.get_full_path`.
                Use ``False`` to not include an action attribute.
            method: The method attribute of the HTML form.
                Defaults to :meth:`.get_default_method` if not specified.
            messages_container: A :class:`django_cradmin.uicontainer.container.AbstractContainerRenderable`
                object where we add and render messages such as global errors and info messages.
                Defaults to :meth:`.get_default_messages_container`.

            **kwargs: Kwargs for :class:`django_cradmin.uicontainer.container.AbstractContainerRenderable`.
        """
        self.form = form
        self.action = action
        self._overridden_method = method
        self.messages_container = messages_container or self.get_default_messages_container()
        self._fieldrenderable_map = {}
        super(Form, self).__init__(**kwargs)
        self.properties['formrenderable'] = self

    def get_default_dom_id(self):
        return 'id_form'

    def prepopulate_children_list(self):
        if self.messages_container:
            return [
                self.messages_container
            ]
        else:
            return []

    def get_default_html_tag(self):
        return 'form'

    def get_default_method(self):
        """
        Get the default value for the method attribute of the form element.

        Defaults to ``"POST"``.
        """
        return 'POST'

    @property
    def method(self):
        """
        Get the value for the method attribute of the form element.

        You should not override this. Override :meth:`.get_default_method` instead.
        """
        return self._overridden_method or self.get_default_method()

    def action_is_redirect_to_self(self):
        return self.action is None

    def get_html_element_attributes(self):
        html_element_attributes = super(Form, self).get_html_element_attributes()
        html_element_attributes['method'] = self.method
        if not self.action_is_redirect_to_self():
            # if action is ``None`` (I.E.: redirect to self), we add this attribute in the
            # template, so it is not added here.
            # We have to do it this way because we do not have access to the request
            # object here.
            html_element_attributes['action'] = self.action
        return html_element_attributes

    def register_field_wrapper_renderable(self, field_wrapper_renderable):
        """
        Register a :class:`~django_cradmin.uicontainer.fieldwrapper.FieldWrapper`
        with this form.

        The form can access all its fields via :meth:`.get_field_wrapper_renderable`
        and :meth:`.iter_field_wrapper_renderables`.

        Args:
            field_wrapper_renderable: A :class:`~django_cradmin.uicontainer.fieldwrapper.FieldWrapper`
                object.
        """
        self._fieldrenderable_map[field_wrapper_renderable.fieldname] = field_wrapper_renderable

    def get_field_wrapper_renderable(self, fieldname):
        """
        Get a :class:`~django_cradmin.uicontainer.fieldwrapper.FieldWrapper`
        that is a child of the form.

        Args:
            fieldname (str): A fieldname for a FieldWrapper that is a child
                of the form. Does not need to be a direct child.

        Raises:
            KeyError: If no FieldWrapper with the provided ``fieldname`` exists
                within the form.

        Returns:
            The FieldWrapper with the provided ``fieldname``.
        """
        return self._fieldrenderable_map[fieldname]

    def iter_field_wrapper_renderables(self):
        """
        Get an iterator over all the
        :class:`~django_cradmin.uicontainer.fieldwrapper.FieldWrapper`
        objects within the form.
        """
        return self._fieldrenderable_map.values()

    def get_default_messages_container(self):
        """
        Get the default message container. Used unless
        the ``messages_container`` kwarg for :meth:`.__init__` overrides it.

        Defaults to a :class:`django_cradmin.uicontainer.messagecontainer.MessagesContainer`.

        Must implement :class:`django_cradmin.uicontainer.messagecontainer.AbstractMessageListMixin`.
        """
        return messagescontainer.MessagesContainer(
            test_css_class_suffixes_list=['form-globalmessages']
        )

    def validate(self):
        if self.form.is_valid():
            self.form_valid()
        else:
            self.form_invalid()

    def form_valid(self):
        pass

    def add_global_form_validation_errors(self, validationerror_list):
        self.messages_container.add_validationerror_list(
            validationerror_list=validationerror_list)

    def add_field_validation_errors(self, field_wrapper_renderable, validationerror_list):
        field_wrapper_renderable.messages_container.add_validationerror_list(
            validationerror_list=validationerror_list)

    def add_field_validation_errors_field_not_child(self, fieldname, validationerror_list):
        field_label = self.form.fields[fieldname].label
        if not field_label:
            field_label = fieldname
        self.messages_container.add_validationerror_list(
            validationerror_list=validationerror_list,
            prefix=field_label)

    def form_invalid(self):
        for fieldname, validationerror_list in self.form.errors.as_data().items():
            if fieldname == '__all__':
                self.add_global_form_validation_errors(validationerror_list=validationerror_list)
            else:
                try:
                    field_wrapper_renderable = self.get_field_wrapper_renderable(fieldname=fieldname)
                except KeyError:
                    self.add_field_validation_errors_field_not_child(
                        fieldname=fieldname,
                        validationerror_list=validationerror_list)
                else:
                    self.add_field_validation_errors(
                        field_wrapper_renderable=field_wrapper_renderable,
                        validationerror_list=validationerror_list)

    def bootstrap(self, **kwargs):
        result = super(Form, self).bootstrap(**kwargs)
        self.validate()
        return result



import json

import os
from setuptools import setup, find_packages

with open(os.path.join(os.path.dirname(__file__), 'django_cradmin', 'version.json')) as f:
    version = json.loads(f.read())


setup(
    name='django_cradmin',
    description='A role based admin UI for Django that produces a user friendly and beautiful UI.',
    version=version,
    url='https://github.com/appressoas/django_cradmin',
    author='Espen Angell Kristiansen, Tor Johansen, Vegard Angell, Magne Westlie',
    author_email='post@appresso.no',
    license='BSD',
    packages=find_packages(
        exclude=[
            'ez_setup',
            'tasks'
        ]),
    zip_safe=False,
    include_package_data=True,
    install_requires=[
        'setuptools',
        'django-crispy-forms>=1.6.0',
        'Django>=1.8',
        'django-multiupload',
        'Jinja2',
        'pytz',
        'future',
        'html2text',
        'htmls',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved',
        'Operating System :: OS Independent',
        'Programming Language :: Python'
    ]
)



import urllib.parse

from django.utils.translation import ugettext_lazy
from django.views.generic import CreateView as DjangoCreateView
from django_cradmin import javascriptregistry

from . import create_update_view_mixin


class CreateViewMixin(create_update_view_mixin.CreateUpdateViewMixin):

    #: The viewname within this app for the edit view.
    #: See :meth:`.get_editurl`.
    editview_appurl_name = 'edit'

    def get_pagetitle(self):
        """
        Get the page title (the title tag).

        Defaults to ``Create <verbose_name model>``.
        """
        return ugettext_lazy('Create %(what)s') % {'what': self.model_verbose_name}

    def get_success_message(self, obj):
        """
        Defaults to ``"Created "<str(obj)>".``
        """
        return ugettext_lazy('Created "%(object)s"') % {'object': obj}

    def get_editurl(self, obj):
        """
        Get the edit URL for ``obj``.

        Defaults to::

            self.request.cradmin_app.reverse_appurl(self.editview_appurl_name, args=[obj.pk])

        You normally want to use :meth:`.get_full_editurl` instead of this method.
        """
        return self.request.cradmin_app.reverse_appurl(self.editview_appurl_name, args=[obj.pk])

    def get_full_editurl(self, obj):
        """
        Get the full edit URL for the provided object.

        Unlike :meth:`.get_editurl`, this ensures that any ``success_url`` in ``request.GET``
        is included in the URL.

        Args:
            obj: A saved model object.
        """
        url = self.get_editurl(obj)
        if 'success_url' in self.request.GET:
            url = '{}?{}'.format(
                url, urllib.parse.urlencode({
                    'success_url': self.request.GET['success_url']}))
        return url


class WithinRoleCreateView(CreateViewMixin,
                           DjangoCreateView,
                           javascriptregistry.viewmixin.WithinRoleViewMixin):
    template_name = 'django_cradmin/viewhelpers/formview/within_role_create_view.django.html'

    def get_context_data(self, **kwargs):
        context = super(WithinRoleCreateView, self).get_context_data(**kwargs)
        self.add_javascriptregistry_component_ids_to_context(context=context)
        self.add_create_update_view_mixin_context_data(context=context)
        return context



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import inspect

from ievv_opensource.utils.singleton import Singleton

from django_cradmin.javascriptregistry.component import AbstractJsComponent


class DuplicateComponentId(Exception):
    """
    Raised by :meth:`.Registry.add` if adding a component_id that is
    already in the registry.
    """


class Registry(Singleton):
    """
    Registry of :class:`.JsComponent` objects.

    Examples:

        First, define a subclass of :class:`~django_cradmin.javascriptregistry.component.AbstractJsComponent`.
        Then register the javascript component in the registry via an AppConfig for your
        Django app::

            from django.apps import AppConfig
            from django_cradmin import javascriptregistry

            class MyJsComponent(javascriptregistry.component.AbstractJsComponent):
                @classmethod
                def get_component_id(cls):
                    return 'my_cool_app'

                def get_head_sourceurls(self):
                    return [
                        self.get_static_url('myapp/js/myapp.js')
                    ]

            class MyAppConfig(AppConfig):
                name = 'myapp'

                def ready(self):
                    javascriptregistry.Registry.get_instance().add(jscomponent_class=MyJsComponent)
    """

    def __init__(self):
        super(Registry, self).__init__()
        self._jscomponent_classes = {}

    def add(self, jscomponent_class):
        """
        Add the given ``jscomponent_class`` to the registry.

        Parameters:
            jscomponent_class (django_cradmin.javascriptregistry.component.AbstractJsComponent):
                The javascript component class to add
                to the registry.
        """
        if jscomponent_class.get_component_id() in self._jscomponent_classes:
            raise DuplicateComponentId('Duplicate component ID in javascriptregistry: {}'.format(
                jscomponent_class.get_component_id()))
        self._jscomponent_classes[jscomponent_class.get_component_id()] = jscomponent_class

    def __contains__(self, component_id_or_object):
        """
        Check if the provided component ID, component object or
        component class is in the registry.

        Args:
            component_id_or_object: Can be a
                :class:`~django_cradmin.javascriptregistry.component.AbstractJsComponent` object,
                or a string.
        """
        if inspect.isclass(component_id_or_object) and issubclass(component_id_or_object, AbstractJsComponent):
            component_id = component_id_or_object.get_component_id()
        else:
            component_id = component_id_or_object
        return component_id in self._jscomponent_classes

    def __getitem__(self, component_id):
        """
        Get the :class:`~django_cradmin.javascri~ptregistry.component.AbstractJsComponent`
        class registered with ``component_id``.
        """
        return self._jscomponent_classes[component_id]

    def _get_component_object(self, request, component_id):
        component_class = self._jscomponent_classes[component_id]
        component = component_class(request=request)
        return component

    def get_component_objects(self, request, component_ids):
        """
        Get the :class:`~django_cradmin.javascri~ptregistry.component.AbstractJsComponent`
        objects for all the provided ``component_ids``, including all their
        dependencies (see :class:`~django_cradmin.javascri~ptregistry.component.AbstractJsComponent.get_dependencies`).

        Args:
            request: A :class:`django.http.HttpRequest` object.
            component_ids: An iterable of component IDs.

        Raises:
            KeyError: If any of the component IDs is not in the registry.

        Returns:
            list: A list of component objects.
        """
        added_component_ids = set()
        components = []

        for requested_component_id in component_ids:
            if requested_component_id in added_component_ids:
                continue

            requested_component = self._get_component_object(
                request=request, component_id=requested_component_id)
            for dependency_component_id in requested_component.get_dependencies():
                if dependency_component_id in added_component_ids:
                    continue
                dependency_component = self._get_component_object(
                    request=request, component_id=dependency_component_id)
                components.append(dependency_component)
                added_component_ids.add(dependency_component_id)

            components.append(requested_component)
            added_component_ids.add(requested_component_id)

        return components


class MockableRegistry(Registry):
    """
    A non-singleton version of :class:`.Registry`. For tests.

    Typical usage in a test::

        from django_cradmin import javascriptregistry

        class MockJsComponent(javascriptregistry.component.AbstractJsComponent):
            @classmethod
            def get_component_id(cls):
                return 'mock_js_component'

        mockregistry = javascriptregistry.MockableRegistry()
        mockregistry.add(MockJsComponent)

        with mock.patch('django_cradmin.javascriptregistry.registry.Registry.get_instance',
                        lambda: mockregistry):
            pass  # ... your code here ...
    """

    def __init__(self):
        self._instance = None  # Ensure the singleton-check is not triggered
        super(MockableRegistry, self).__init__()



from .registry import DuplicateComponentId  # noqa
from .registry import Registry  # noqa
from .registry import MockableRegistry  # noqa
from . import component  # noqa
from . import viewmixin  # noqa



from __future__ import unicode_literals
from django.template import Library
from django_cradmin import javascriptregistry
from django_cradmin.utils import cradmin_collections

register = Library()


class JavascriptRegistryTemplateTagError(Exception):
    pass


def _get_components_from_context(context):
    if 'cradmin_javascriptregistry_component_ids' not in context:
        raise JavascriptRegistryTemplateTagError(
            'The cradmin_javascriptregistry_component_ids variable is not in the '
            'template context.')  # TODO mention the view class mixin
    if 'request' not in context:
        raise JavascriptRegistryTemplateTagError(
            'The request variable is not in the template context. '
            'You can get this using the "django.template.context_processors.request" context processor.')
    component_ids = context['cradmin_javascriptregistry_component_ids']
    request = context['request']
    registry = javascriptregistry.Registry.get_instance()
    components = registry.get_component_objects(request=request, component_ids=component_ids)
    return components


def _get_sourceurls_from_components(components, methodname):
    all_sourceurls = cradmin_collections.OrderedSet()
    for component in components:
        sourceurls = getattr(component, methodname)()
        for sourceurl in sourceurls:
            all_sourceurls.add(sourceurl)
    return all_sourceurls


def _get_javascript_code_from_components(components, methodname):
    javascript_code_list = []
    for component in components:
        javascript_code = getattr(component, methodname)()
        if javascript_code:
            javascript_code = javascript_code.strip()
            if javascript_code:
                javascript_code_list.append(javascript_code)
    all_javascript_code = '\n'.join(javascript_code_list)
    return all_javascript_code


@register.inclusion_tag('django_cradmin/templatetags/javascriptregistry/script_tags_from_sourceurls.django.html',
                        takes_context=True)
def cradmin_javascriptregistry_head(context):
    """
    Renders the jsregistry code for the ``<head>`` tag.

    Raises:

    """
    return {
        'sourceurls': _get_sourceurls_from_components(
            components=_get_components_from_context(context=context),
            methodname='get_head_sourceurls')
    }


@register.inclusion_tag(
    'django_cradmin/templatetags/javascriptregistry/end_of_body.django.html',
    takes_context=True)
def cradmin_javascriptregistry_end_of_body(context):
    """
    Renders the jsregistry code for the end of the ``<body>`` tag.
    """
    components = _get_components_from_context(context=context)
    return {
        'sourceurls': _get_sourceurls_from_components(
            components=components,
            methodname='get_sourceurls'),
        'javascript_code_before_sourceurls': _get_javascript_code_from_components(
            components=components,
            methodname='get_javascript_code_before_sourceurls'),
        'javascript_code_after_sourceurls': _get_javascript_code_from_components(
            components=components,
            methodname='get_javascript_code_after_sourceurls'),
    }



from __future__ import unicode_literals
from django.contrib.auth import get_user_model


def create_testuser(username=None, password='test', **kwargs):
    user = get_user_model()(username=username, **kwargs)
    user.set_password(password)
    user.save()
    return user



import htmls
from django import test
from django_cradmin import uicontainer


class TestPageSection(test.TestCase):
    def test_sanity(self):
        container = uicontainer.layout.AdminuiPageSection().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('.adminui-page-section'))

    def test_default_css_classes(self):
        container = uicontainer.layout.AdminuiPageSection(dom_id='id_pagesection').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('#id_pagesection').cssclasses_set,
                         {'adminui-page-section', 'test-adminui-page-section'})


class TestPageSectionTight(test.TestCase):
    def test_sanity(self):
        container = uicontainer.layout.AdminuiPageSectionTight().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('.adminui-page-section--tight'))

    def test_default_css_classes(self):
        container = uicontainer.layout.AdminuiPageSectionTight(dom_id='id_pagesection').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('#id_pagesection').cssclasses_set,
                         {'adminui-page-section', 'adminui-page-section--tight', 'test-adminui-page-section'})


class TestContainer(test.TestCase):
    def test_sanity(self):
        container = uicontainer.layout.Container().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('.container'))

    def test_default_css_classes(self):
        container = uicontainer.layout.Container(dom_id='id_container').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('#id_container').cssclasses_set,
                         {'container', 'test-container'})


class TestContainerTight(test.TestCase):
    def test_sanity(self):
        container = uicontainer.layout.ContainerTight().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('.container--tight'))

    def test_default_css_classes(self):
        container = uicontainer.layout.ContainerTight(dom_id='id_container').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('#id_container').cssclasses_set,
                         {'container', 'container--tight', 'test-container'})



from django.views.generic import FormView as DjangoFormView
from django_cradmin import javascriptregistry

from . import formviewmixin


class WithinRoleFormView(javascriptregistry.viewmixin.WithinRoleViewMixin,
                         formviewmixin.FormViewMixin,
                         DjangoFormView):
    """
    Form view.

    Uses :class:`django.views.generic.edit.FormView` with
    :class:`django_cradmin.viewhelpers.formview.formviewmixin.FormViewMixin`
    and :class:`django_cradmin.javascriptregistry.viewmixin.WithinRoleViewMixin`.

    Examples:

        Minimalistic example::

            from django import forms
            from django.http import HttpResponseRedirect
            from django_cradmin.viewhelpers import formview

            class MyForm(forms.Form):
                first_name = forms.CharField(max_length=50)
                last_name = forms.CharField(max_length=50)

            class MyFormView(formview.WithinRoleFormView):
                template_name = 'myapp/myview.django.html'
                form_class = MyForm

                def get_form_renderable(self):
                    return uicontainer.layout.AdminuiPageSectionTight(
                        children=[
                            uicontainer.form.Form(
                                form=self.get_form(),
                                children=[
                                    uicontainer.fieldwrapper.FieldWrapper(
                                        fieldname='first_name',
                                        # Override field renderable to set autofocus
                                        field_renderable=uicontainer.field.Field(autofocus=True)
                                    ),
                                    uicontainer.fieldwrapper.FieldWrapper('last_name'),
                                    uicontainer.button.SubmitPrimary(text='Save')
                                ]
                            )
                        ]
                    ).bootstrap()

                def form_valid(self, form):
                    # ... do something with the form ...
                    return HttpResponseRedirect('/some/view')
    """
    template_name = 'django_cradmin/viewhelpers/formview/within_role_form_view.django.html'

    def get_context_data(self, **kwargs):
        context = super(WithinRoleFormView, self).get_context_data(**kwargs)
        self.add_formview_mixin_context_data(context=context)
        self.add_javascriptregistry_component_ids_to_context(context=context)
        return context


class StandaloneFormView(javascriptregistry.viewmixin.StandaloneBaseViewMixin,
                         formviewmixin.FormViewMixin,
                         DjangoFormView):
    template_name = 'django_cradmin/viewhelpers/formview/standalone_form_view.django.html'

    def get_context_data(self, **kwargs):
        context = super(StandaloneFormView, self).get_context_data(**kwargs)
        self.add_formview_mixin_context_data(context=context)
        self.add_javascriptregistry_component_ids_to_context(context=context)
        return context



from django.forms import widgets


class WrappedSelect(widgets.Select):
    def __init__(self, *args, **kwargs):
        self.wrapper_css_class = kwargs.pop('wrapper_css_class', '')
        super(WrappedSelect, self).__init__(*args, **kwargs)

    def render(self, *args, **kwargs):
        rendered_widget = super(WrappedSelect, self).render(*args, **kwargs)
        return u'<div class="{}">{}</div>'.format(
            self.wrapper_css_class,
            u''.join(rendered_widget))



from __future__ import unicode_literals

from django.conf.urls import url

from django_cradmin.apps.cradmin_authenticate.views.login import LoginView
from django_cradmin.apps.cradmin_authenticate.views.logout import cradmin_logoutview

urlpatterns = [
    url(r'^login$', LoginView.as_view(), name='cradmin-authenticate-login'),
    url(r'^logout$', cradmin_logoutview, name='cradmin-authenticate-logout'),
]



from __future__ import unicode_literals

from django.conf.urls import url

from django_cradmin.apps.cradmin_register_account.views.begin import BeginRegisterAccountView
from django_cradmin.apps.cradmin_register_account.views.email_sent import EmailSentView

urlpatterns = [
    url(r'^begin',
        BeginRegisterAccountView.as_view(),
        name="cradmin-register-account-begin"),
    url(r'^email-sent',
        EmailSentView.as_view(),
        name="cradmin-register-account-email-sent"),
]



from __future__ import unicode_literals

import htmls
import mock
from django import forms
from django_cradmin import uicontainer


class SingleFormRenderableHelperMixin(object):
    def single_field_formclass_factory(self, field):
        class TestForm(forms.Form):
            testfield = field
        return TestForm

    def single_field_formrenderable_fieldwrapper_factory(self):
        return uicontainer.fieldwrapper.FieldWrapper(fieldname='testfield')

    def single_field_formrenderable_factory(self, field, formvalue=None,
                                            fieldwrapper=None):
        form_class = self.single_field_formclass_factory(field=field)
        data = None
        if formvalue:
            data = {'testfield': formvalue}
        form = form_class(data=data)
        fieldwrapper = fieldwrapper or self.single_field_formrenderable_fieldwrapper_factory()
        formrenderable = uicontainer.form.Form(
            form=form,
            children=[fieldwrapper]
        ).bootstrap()
        return formrenderable

    def single_field_formrenderable_htmls(self, **kwargs):
        formrenderable = self.single_field_formrenderable_factory(**kwargs)
        return htmls.S(formrenderable.render(request=mock.MagicMock()))



from __future__ import unicode_literals

from . import form_mixins
from . import container


class BaseHelpTextRenderable(container.AbstractContainerRenderable, form_mixins.FieldWrapperRenderableChildMixin):
    """
    Base class for renderers of help text for a
    :class:`~django_cradmin.uicontainer.fieldwrapper.FieldWrapper`.

    You never use this on its own outside a
    :class:`~django_cradmin.uicontainer.fieldwrapper.FieldWrapper`.
    """

    def get_default_bem_block_or_element(self):
        return 'formfield-help-text'


class AutomaticHelpText(BaseHelpTextRenderable):
    """
    Renders help text for a :class:`~django_cradmin.uicontainer.fieldwrapper.FieldWrapper`
    using the help text from the Django form field.

    You never use this on its own outside a
    :class:`~django_cradmin.uicontainer.fieldwrapper.FieldWrapper`.
    """
    template_name = 'django_cradmin/uicontainer/field_help_text/automatic_help_text.django.html'

    def get_default_html_tag(self):
        return 'p'

    @property
    def help_text(self):
        """
        Get the help text configured for the form field.

        Can be overridden to provide a custom help text.
        """
        bound_formfield = self.field_wrapper_renderable.bound_formfield
        return bound_formfield.help_text

    @property
    def should_render(self):
        """
        Returns ``False`` if :meth:`.help_text` is empty,
        which means that the help text is not rendered at all if
        there is no help text.
        """
        return bool(self.help_text)



import json

import htmls
from django.conf import settings
from django.contrib.auth.models import AnonymousUser
from django.test import RequestFactory
from django_cradmin.python2_compatibility import mock
from model_mommy import mommy


def _indent_string(string):
    try:
        return '\n'.join(['   {}'.format(line) for line in string.split('\n')])
    except:
        return string


class MockRequestResponse(object):
    """
    Return type of :meth:`.TestCaseMixin.mock_request`,
    :meth:`.TestCaseMixin.mock_http200_getrequest_htmls` and
    :meth:`.TestCaseMixin.mock_postrequest`.

    .. attribute:: response

        The HttpResponse object.

    .. attribute:: request

        The RequestFactory-generated HttpRequest object.

    .. attribute:: selector

        A ``htmls.S`` object created with the entire content of the
        response as input. Only available when using
        :meth:`.TestCaseMixin.mock_http200_getrequest_htmls`.
    """
    def __init__(self, response, request):
        self.response = response
        self.request = request
        self.selector = None


class AbstractTestCaseMixin(object):
    """
    Implements the common API for the test case mixins.
    """

    #: The view class - must be set in subclasses unless
    #: you override :meth:`~.AbstractTestCaseMixin.get_viewclass`.
    viewclass = None

    def get_viewclass(self):
        """
        Get the view class. Defaults to :obj:`.~AbstractTestCaseMixin.viewclass`
        """
        return self.viewclass

    def make_view(self):
        """
        Make view function.

        Calls ``get_viewclass().as_view()`` by default.
        """
        viewclass = self.get_viewclass()
        if self.viewclass is None:
            raise NotImplementedError('You must set the viewclass attribute on TestCase classes using TestCaseMixin.')
        return viewclass.as_view()

    def get_requestfactory_class(self):
        """
        Get the request factory class.

        Must be implemented in subclasses.
        """
        raise NotImplementedError()

    def make_requestfactory(self):
        return self.get_requestfactory_class()()

    def create_default_user_for_mock_request(self):
        """
        Create default user for mock request.

        Defaults to returning ``mommy.make(settings.AUTH_USER_MODEL)``,
        which should create a user no matter what user model you
        are using, unless you do something very complex for users.

        You can override this if you want to change the default
        properties of users for all tests in this testcase, but
        it is normally better to just use the ``requestuser`` kwarg
        for :meth:`.mock_request` (or any of the more specialized
        ``mock_*request(..)`` methods)

        Must return a user object compatible with ``django.http.HttpRequest.user``.
        This means that the method can also return an AnonymousUser, but if you need this
        you should use :class:`.NoLoginTestCaseMixin` (which just overrides this method).
        """
        return mommy.make(settings.AUTH_USER_MODEL)

    def make_minimal_request(self, method, requestkwargs=None):
        """
        Make a minimal request object.

        Args:
            method (str): The http method (get, post, ...).
            requestkwargs: (dict): Kwargs for the request. Defaults to
                ``{'path': '/'}``, and the path is added unless it is
                included in the dict. The data you can include in this
                dict is the same as for the get, post, ... methods on
                :class:`.django.test.Client` (depends on the ``method`` kwarg).

        Returns:
            The created request object.
        """
        requestkwargs_full = {
            'path': '/'
        }
        if requestkwargs:
            requestkwargs_full.update(requestkwargs)
        request = getattr(self.make_requestfactory(), method)(**requestkwargs_full)
        return request

    def add_essential_cradmin_attributes_to_request(self, request, requestuser, cradmin_role, cradmin_app,
                                                    cradmin_instance, sessionmock, messagesmock):
        request.user = requestuser or self.create_default_user_for_mock_request()
        request.cradmin_role = cradmin_role or mock.MagicMock()
        request.cradmin_app = cradmin_app or mock.MagicMock()
        request.cradmin_instance = cradmin_instance or mock.MagicMock()
        request.session = sessionmock or mock.MagicMock()
        request._messages = messagesmock or mock.MagicMock()
        return request

    def __add_selector_to_mockresponse(self, mockresponse):
        mockresponse.response.render()
        mockresponse.selector = htmls.S(mockresponse.response.content)

    def prettyformat_response_content(self, response):
        """
        Must be implemented in subclasses.
        """
        raise NotImplementedError()

    def prettyformat_response_meta(self, response):
        return (
            'status_code={status_code}\n'
            'charset={charset}\n'
            'reason_phrase={reason_phrase}'
        ).format(
            status_code=response.status_code,
            charset=response.charset,
            reason_phrase=response.reason_phrase,)

    def prettyformat_response(self, response):
        meta = self.prettyformat_response_meta(response)
        content, warnings = self.prettyformat_response_content(response)
        return '[cradmin TestCaseMixin info]: HttpRequest meta:\n{meta}\n{warnings}\n{content}'.format(
            meta=_indent_string(meta),
            warnings='\n'.join(warnings),
            content=content
        )

    def assert_status_code_of_response(self, response, expected_statuscode):
        errormessage = 'Expected HTTP response with status code {expected}. Got {actual}.'.format(
            expected=expected_statuscode,
            actual=response.status_code
        )
        self.assertEqual(response.status_code, expected_statuscode,
                         msg=errormessage)

    def mock_request(self, method,
                     cradmin_role=None,
                     cradmin_app=None,
                     cradmin_instance=None,
                     requestuser=None,
                     messagesmock=None,
                     sessionmock=None,
                     requestattributes=None,
                     requestkwargs=None,
                     viewkwargs=None,
                     expected_statuscode=None,
                     htmls_selector=False,
                     verbose=False):
        """
        Create a mocked request using :meth:`.make_requestfactory` and :class:`mock.MagicMock`.

        Parameters:
            method: The http method (get, post, ...).
            requestkwargs: (dict): Forwarded to :meth:`.make_minimal_request`.

            cradmin_role: The request.cradmin_role to use. Defaults to mock.MagicMock().
            cradmin_app: The request.cradmin_app to use. Defaults to mock.MagicMock().
            cradmin_instance: The request.cradmin_instance to use. Defaults to mock.MagicMock().
            requestuser: The request.requestuser to use. Defaults to mock.MagicMock().
            sessionmock: The request.session to use. Defaults to mock.MagicMock().
            messagesmock: The request._messages to use. Defaults to mock.MagicMock().

            requestattributes (dict): Extra attributes to the reques to object.
                This is applied before :meth:`.add_essential_cradmin_attributes_to_request`,
                so any of the attributes that method sets can not be in this dics.
            viewkwargs (dict): Kwargs for the view.
            expected_statuscode (int): Expected status code. If this is ``None``,
                we do not validate the status code. Defaults to ``None``.
            htmls_selector (boolean): If this is ``True``, we create a ``htmls.S()``
                object for the response content. Should normally not be used without
                also providing ``expected_statuscode`` because it can lead to
                unexpected behavior. Defaults to ``False``.
            verbose (boolean): More verbose exception messages. Useful for debugging,
                but should be ``False`` when you are not debugging. Defaults
                to ``False``.
        """
        request = self.make_minimal_request(method=method,
                                            requestkwargs=requestkwargs)
        if requestattributes:
            for key, value in requestattributes.items():
                setattr(request, key, value)
        self.add_essential_cradmin_attributes_to_request(
            request=request,
            requestuser=requestuser,
            cradmin_role=cradmin_role,
            cradmin_app=cradmin_app,
            cradmin_instance=cradmin_instance,
            sessionmock=sessionmock,
            messagesmock=messagesmock
        )
        viewkwargs = viewkwargs or {}
        response = self.make_view()(request, **viewkwargs)

        if verbose:
            print(self.prettyformat_response(response=response))

        if expected_statuscode is not None:
            self.assert_status_code_of_response(
                response=response, expected_statuscode=expected_statuscode)
        mockresponse = MockRequestResponse(response=response, request=request)
        if htmls_selector:
            self.__add_selector_to_mockresponse(mockresponse)
        return mockresponse

    def mock_getrequest(self, **kwargs):
        return self.mock_request(method='get', **kwargs)

    def mock_http302_getrequest(self, **kwargs):
        kwargs['expected_statuscode'] = 302
        return self.mock_getrequest(**kwargs)

    def mock_postrequest(self, **kwargs):
        return self.mock_request(method='post', **kwargs)

    def mock_http302_postrequest(self, **kwargs):
        kwargs['expected_statuscode'] = 302
        return self.mock_postrequest(**kwargs)

    def mock_http200_postrequest(self, **kwargs):
        kwargs['expected_statuscode'] = 200
        return self.mock_postrequest(**kwargs)

    def mock_http201_postrequest(self, **kwargs):
        kwargs['expected_statuscode'] = 201
        return self.mock_postrequest(**kwargs)

    def mock_http400_postrequest(self, **kwargs):
        kwargs['expected_statuscode'] = 400
        return self.mock_postrequest(**kwargs)

    def mock_http403_postrequest(self, **kwargs):
        kwargs['expected_statuscode'] = 403
        return self.mock_postrequest(**kwargs)

    def mock_putrequest(self, **kwargs):
        return self.mock_request(method='put', **kwargs)

    def mock_patchrequest(self, **kwargs):
        return self.mock_request(method='patch', **kwargs)

    def mock_deleterequest(self, **kwargs):
        return self.mock_request(method='delete', **kwargs)


class TestCaseMixin(AbstractTestCaseMixin):
    """
    A mixin class that makes it easier to write tests for cradmin views.

    It mocks all the required attributes of the request object.

    Examples:

        Minimalistic::

            from django_cradmin import cradmin_testhelpers

            class TestMyView(TestCase, cradmin_testhelpers.TestCaseMixin):
                viewclass = MyView

                def test_get_render_form(self):
                    mockresponse = self.mock_http200_getrequest_htmls()
                    mockresponse.selector.prettyprint()

                def test_post(self):
                    mockresponse = self.mock_postrequest(requestkwargs={
                        'data': {
                            'name': 'Jane Doe',
                            'age': 24
                        }
                    })
                    self.assertEqual(mockresponse.response.status_code, 302)


        Views that take arguments::

            from django_cradmin import cradmin_testhelpers

            class TestMyView(TestCase, cradmin_testhelpers.TestCaseMixin):
                viewclass = MyView

                def test_get_render_form(self):
                    # The kwargs for mock_postrequest, mock_http200_getrequest_htmls
                    # and mock_getrequest are the same, so only showing one for brevity.
                    mockresponse = self.mock_http200_getrequest_htmls(viewkwargs={'pk': 10})

        Views that use a querystring (GET)::

            from django_cradmin import cradmin_testhelpers

            class TestMyView(TestCase, cradmin_testhelpers.TestCaseMixin):
                viewclass = MyView

                def test_get_render_form(self):
                    mockresponse = self.mock_http200_getrequest_htmls(
                        requestkwargs={
                            'data': {
                                'orderby': 'name'
                            }
                        }
                    )

        Using a real user object::

            from django_cradmin import cradmin_testhelpers

            class TestMyView(TestCase, cradmin_testhelpers.TestCaseMixin):
                viewclass = MyView

                def test_post(self):
                    requestuser = mommy.make(settings.AUTH_USER_MODEL)
                    mockresponse = self.mock_http200_getrequest_htmls(requestuser=requestuser)

        Mocking Django messages framework messages::

            from django_cradmin import cradmin_testhelpers

            class TestMyView(TestCase, cradmin_testhelpers.TestCaseMixin):
                viewclass = MyView

                def test_post(self):
                    messagesmock = mock.MagicMock()
                    mockresponse = self.mock_postrequest(
                        requestkwargs={
                            'data': {
                                'name': 'Jane Doe',
                                'age': 24
                        },
                        messagesmock=messagesmock
                    )
                    messagesmock.add.assert_called_once_with(
                        messages.SUCCESS,
                        'The data was posted successfully!',
                        '')

        Get a bit of extra information useful while debugging
        failing tests - use ``verbose=True`` (only for debugging, do
        not keep/commit verbose=True)::

            from django_cradmin import cradmin_testhelpers

            class TestMyView(TestCase, cradmin_testhelpers.TestCaseMixin):
                viewclass = MyView

                def test_get(self):
                    mockresponse = self.mock_http200_getrequest_htmls(
                        verbose=True
                    )

    """

    def get_requestfactory_class(self):
        """
        Get the request factory class. Defaults to :class:`django.test.RequestFactory`.
        """
        return RequestFactory

    def prettyformat_response_content(self, response):
        warnings = []
        output = None
        if hasattr(response, 'render'):
            try:
                response.render()
            except Exception as e:
                warnings.append('[cradmin TestCaseMixin warning] response.render() failed with: {}'.format(e))
            else:
                try:
                    output = '[cradmin TestCaseMixin info]: Prettyformatted response.content:\n{}'.format(
                        _indent_string(htmls.S(response.content).prettify())
                    )
                except:
                    pass
        if output is None:
            try:
                content = response.content.decode('utf-8')
            except UnicodeError:
                content = response.content
            if content:
                output = '[cradmin TestCaseMixin info]: response.content:\n{}'.format(
                    _indent_string(content))
            else:
                output = '[cradmin TestCaseMixin info]: response.content is empty.'
        return output, warnings

    def mock_http200_getrequest_htmls(self, **kwargs):
        kwargs['expected_statuscode'] = 200
        kwargs['htmls_selector'] = True
        return self.mock_getrequest(**kwargs)

    def mock_http200_postrequest_htmls(self, **kwargs):
        kwargs['expected_statuscode'] = 200
        kwargs['htmls_selector'] = True
        return self.mock_postrequest(**kwargs)


class NoLoginTestCaseMixin(TestCaseMixin):
    """
    Use this instead of :class:`.TestCaseMixin` if you do not require
    an authenticated user in your view.
    """
    def create_default_user_for_mock_request(self):
        """
        Overridden to return a :class:`django.contrib.auth.models.AnonymousUser`
        object instead of creating a user.
        """
        return AnonymousUser()


class RestFrameworkApiTestCaseMixin(TestCaseMixin):
    """
    Django-rest-framework API test case mixin.

    Works just like :class:`.TestCaseMixin`, except that it
    has a different debug output, and no ``*_htmls`` methods.

    Assumes that the viewclass is a subclass of :class:`rest_framework.views.APIView`.

    You can also make it work with viewsets by overriding :meth:`~.AbstractTestCaseMixin.make_view`
    and use ``return ViewSetClass.as_view({...})`` there.
    """
    def get_requestfactory_class(self):
        # Importing it here to avoid har dependency on rest framework.
        from rest_framework.test import APIRequestFactory
        return APIRequestFactory

    def prettyformat_response_content(self, response):
        warnings = []
        output = None
        if hasattr(response, 'data'):
            output = _indent_string(json.dumps(response.data, indent=4))
        else:
            warnings.append('[cradmin RestFrameworkApiTestCaseMixin warning] '
                            'Response has no data attribute.')
        if output is None:
            try:
                content = response.content.decode('utf-8')
            except UnicodeError:
                content = response.content
            if content:
                output = '[cradmin RestFrameworkApiTestCaseMixin info]: response.content:\n{}'.format(
                    _indent_string(content))
            else:
                output = '[cradmin RestFrameworkApiTestCaseMixin info]: response.content is empty.'
        return output, warnings

    def mock_request(self, **kwargs):
        requestkwargs = kwargs.pop('requestkwargs', {})
        requestkwargs['format'] = requestkwargs.get('format', 'json')
        kwargs['requestkwargs'] = requestkwargs
        return super(RestFrameworkApiTestCaseMixin, self).mock_request(**kwargs)



from django.conf.urls import url

from django_cradmin.apps.cradmin_kss_styleguide.views import exampleview
from .views import styleguideview

urlpatterns = [
    url(r'^$', styleguideview.GuideListView.as_view(),
        name='cradmin_kss_styleguide_guides'),
    url(r'^guide/(?P<unique_id>[a-zA-Z_.0-9-]+)/(?P<prefix>[^/]+)?',
        styleguideview.GuideView.as_view(),
        name='cradmin_kss_styleguide_guide'),
    url(r'^example/(?P<unique_id>[a-zA-Z_.0-9-]+)/(?P<section>[^/]+)/(?P<exampleindex>\d+)',
        exampleview.ExampleView.as_view(),
        name='cradmin_kss_styleguide_example'),
]



from django_cradmin.viewhelpers.listbuilder import base


class RowList(base.List):
    def get_test_css_class_suffixes_list(self):
        css_class_suffixes = super(RowList, self).get_test_css_class_suffixes_list()
        css_class_suffixes.append('cradmin-listbuilder-list')
        return css_class_suffixes

    def get_base_css_classes_list(self):
        css_classes = super(RowList, self).get_base_css_classes_list()
        css_classes.append('blocklist')
        return css_classes



import htmls
import mock
from django import forms
from django import test
from django_cradmin import uicontainer


class MinimalForm(forms.Form):
    name = forms.CharField()


class TestFormRenderable(test.TestCase):
    def ttest_example(self):
        class ExampleForm(forms.Form):
            name = forms.CharField()
            age = forms.IntegerField()
            created_by = forms.CharField()
            user_type = forms.ChoiceField(
                choices=[
                    ('a', 'A'),
                    ('b', 'B'),
                ],
                initial='b',
                widget=forms.RadioSelect()
            )

        form = ExampleForm()
        formrenderable = uicontainer.form.Form(
            form=form,
            children=[
                uicontainer.layout.AdminuiPageSectionTight(
                    children=[
                        uicontainer.fieldwrapper.FieldWrapper(
                            fieldname='name',
                            field_renderable=uicontainer.field.Field(
                                autofocus=True,
                                placeholder='The name ffs...'
                            )
                        ),
                        uicontainer.fieldwrapper.FieldWrapper(fieldname='age'),
                        uicontainer.fieldwrapper.FieldWrapper(fieldname='user_type'),
                    ]
                ),
                uicontainer.layout.AdminuiPageSection(
                    bem_variant_list=['supertight'],
                    children=[
                        uicontainer.fieldwrapper.FieldWrapper(fieldname='created_by'),
                        uicontainer.fieldset.FieldSet(
                            title='Metadata',
                            children=[
                                uicontainer.fieldwrapper.FieldWrapper(fieldname='created_by'),
                            ]
                        )
                    ],
                ),
                uicontainer.fieldwrapper.FieldWrapper(
                    fieldname='user_type',
                    field_renderable=uicontainer.field.Field(
                        subwidget_renderable_kwargs={
                            'bem_variant_list': ['inline']
                        }
                    )),
            ]
        ).bootstrap()
        htmls.S(formrenderable.render(request=mock.MagicMock())).prettyprint()

    def test_get_default_method(self):
        formrenderable = uicontainer.form.Form(form=MinimalForm())
        self.assertEqual(formrenderable.get_default_method(), 'POST')

    def test_method_default(self):
        formrenderable = uicontainer.form.Form(form=MinimalForm())
        self.assertEqual(formrenderable.method, 'POST')

    def test_method_override_default(self):
        class MyFormRenderable(uicontainer.form.Form):
            def get_default_method(self):
                return 'GET'
        formrenderable = MyFormRenderable(form=MinimalForm())
        self.assertEqual(formrenderable.method, 'GET')

    def test_method_overridden_with_kwarg(self):
        formrenderable = uicontainer.form.Form(form=MinimalForm(), method='GET')
        self.assertEqual(formrenderable.method, 'GET')

    def test_action_is_redirect_to_self_default(self):
        formrenderable = uicontainer.form.Form(form=MinimalForm())
        self.assertEqual(formrenderable.action_is_redirect_to_self(), True)

    def test_action_is_redirect_to_self_action_kwarg_none(self):
        formrenderable = uicontainer.form.Form(form=MinimalForm(), action=None)
        self.assertEqual(formrenderable.action_is_redirect_to_self(), True)

    def test_action_is_redirect_to_self_action_kwarg_not_none(self):
        formrenderable = uicontainer.form.Form(form=MinimalForm(), action='/test/submit')
        self.assertEqual(formrenderable.action_is_redirect_to_self(), False)

    def test_get_html_element_attributes_method(self):
        formrenderable = uicontainer.form.Form(form=MinimalForm(), method='GET')
        self.assertEqual(formrenderable.get_html_element_attributes()['method'], 'GET')

    def test_get_html_element_attributes_action_default(self):
        formrenderable = uicontainer.form.Form(form=MinimalForm())
        # Not included by default - it is rendered by the template by default because
        # the default uses request.get_full_path()
        self.assertNotIn('action', formrenderable.get_html_element_attributes())

    def test_get_html_element_attributes_action_kwarg(self):
        formrenderable = uicontainer.form.Form(form=MinimalForm(), action='/test/submit')
        self.assertEqual(formrenderable.get_html_element_attributes()['action'], '/test/submit')

    def test_register_field_wrapper_renderable(self):
        mock_field_wrapper_renderable = mock.MagicMock()
        mock_field_wrapper_renderable.fieldname = 'test'
        formrenderable = uicontainer.form.Form(form=MinimalForm())
        formrenderable.register_field_wrapper_renderable(field_wrapper_renderable=mock_field_wrapper_renderable)
        self.assertEqual(formrenderable._fieldrenderable_map['test'], mock_field_wrapper_renderable)

    def test_get_field_wrapper_renderable(self):
        mock_field_wrapper_renderable = mock.MagicMock()
        mock_field_wrapper_renderable.fieldname = 'test'
        formrenderable = uicontainer.form.Form(form=MinimalForm())
        formrenderable.register_field_wrapper_renderable(field_wrapper_renderable=mock_field_wrapper_renderable)
        self.assertEqual(formrenderable.get_field_wrapper_renderable('test'), mock_field_wrapper_renderable)

    def test_iter_field_wrapper_renderables(self):
        mock_field_wrapper_renderable1 = mock.MagicMock()
        mock_field_wrapper_renderable1.fieldname = 'test1'
        mock_field_wrapper_renderable2 = mock.MagicMock()
        mock_field_wrapper_renderable2.fieldname = 'test2'
        formrenderable = uicontainer.form.Form(form=MinimalForm())
        formrenderable.register_field_wrapper_renderable(field_wrapper_renderable=mock_field_wrapper_renderable1)
        formrenderable.register_field_wrapper_renderable(field_wrapper_renderable=mock_field_wrapper_renderable2)
        field_wrapper_renderables = list(formrenderable.iter_field_wrapper_renderables())
        self.assertIn(mock_field_wrapper_renderable1, field_wrapper_renderables)
        self.assertIn(mock_field_wrapper_renderable2, field_wrapper_renderables)

    def __make_mockrequest(self):
        mockrequest = mock.MagicMock()

        def getitem(me, key):
            return {
                'get_full_path': '/request-full-path'
            }.get(key, None)

        mockrequest.__getitem__ = getitem
        return mockrequest

    def test_render_wrappertag(self):
        formrenderable = uicontainer.form.Form(form=MinimalForm()).bootstrap()
        selector = htmls.S(formrenderable.render(request=self.__make_mockrequest()))
        self.assertTrue(selector.exists('form'))

    def test_render_action_default(self):
        formrenderable = uicontainer.form.Form(form=MinimalForm()).bootstrap()
        selector = htmls.S(formrenderable.render(request=self.__make_mockrequest()))
        self.assertEqual(selector.one('form')['action'], '/request-full-path')

    def test_render_action_kwarg_false(self):
        formrenderable = uicontainer.form.Form(
            form=MinimalForm(), action=False).bootstrap()
        selector = htmls.S(formrenderable.render(request=self.__make_mockrequest()))
        self.assertFalse(selector.one('form').hasattribute('action'))

    def test_render_action_kwarg_string(self):
        formrenderable = uicontainer.form.Form(
            form=MinimalForm(), action='/overridden').bootstrap()
        selector = htmls.S(formrenderable.render(request=self.__make_mockrequest()))
        self.assertEqual(selector.one('form')['action'], '/overridden')

    def test_form_invalid_global_error(self):
        class TestForm(forms.Form):
            name = forms.CharField(required=False)

            def clean(self):
                raise forms.ValidationError('Global error test')

        form = TestForm(data={})
        formrenderable = uicontainer.form.Form(form=form).bootstrap()
        selector = htmls.S(formrenderable.render(request=self.__make_mockrequest()))
        self.assertEqual(selector.one('.test-form-globalmessages .test-warning-message').alltext_normalized,
                         'Global error test')

    def test_form_invalid_field_error(self):
        class TestForm(forms.Form):
            name = forms.CharField(required=True)

        form = TestForm(data={})
        formrenderable = uicontainer.form.Form(
            form=form,
            children=[uicontainer.fieldwrapper.FieldWrapper(fieldname='name')]
        ).bootstrap()
        selector = htmls.S(formrenderable.render(request=self.__make_mockrequest()))
        self.assertEqual(selector.one('#id_name_wrapper .test-warning-message').alltext_normalized,
                         'This field is required.')

    def test_form_invalid_field_error_field_not_rendered_without_label(self):
        class TestForm(forms.Form):
            name = forms.CharField(required=True)

        form = TestForm(data={})
        formrenderable = uicontainer.form.Form(form=form).bootstrap()
        selector = htmls.S(formrenderable.render(request=self.__make_mockrequest()))
        self.assertEqual(selector.one('.test-form-globalmessages .test-warning-message').alltext_normalized,
                         'name: This field is required.')

    def test_form_invalid_field_error_field_not_rendered_with_label(self):
        class TestForm(forms.Form):
            name = forms.CharField(required=True, label='The name')

        form = TestForm(data={})
        formrenderable = uicontainer.form.Form(form=form).bootstrap()
        selector = htmls.S(formrenderable.render(request=self.__make_mockrequest()))
        self.assertEqual(selector.one('.test-form-globalmessages .test-warning-message').alltext_normalized,
                         'The name: This field is required.')
